/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.io.File;
import java.io.IOException;
import java.util.Locale;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import org.jdtaus.core.client.util.UserInterface;
import org.jdtaus.core.client.util.spi.DefaultApplication;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Modules;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.logging.spi.Logger;

/**
 * Client application for editing DTAUS files.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: Editor.java 2893 2007-04-15 22:24:41Z schulte2005 $
 */
public class Editor extends DefaultApplication
{
    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(Editor.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Protected <code>Editor</code> implementation constructor.
    * @param meta Implementation meta-data.
    */
    protected Editor(final Implementation meta)
    {
        super(meta);
        Property p;

        p = meta.getProperties().getProperty("directoryName");
        this._directoryName = (java.lang.String) p.getValue();


        p = meta.getProperties().getProperty("logfileName");
        this._logfileName = (java.lang.String) p.getValue();


    }
    /** Protected <code>Editor</code> dependency constructor.
    * @param meta Dependency meta-data.
    */
    protected Editor(final Dependency meta)
    {
        super(meta);
        Property p;

        p = meta.getProperties().getProperty("directoryName");
        this._directoryName = (java.lang.String) p.getValue();


        p = meta.getProperties().getProperty("logfileName");
        this._logfileName = (java.lang.String) p.getValue();


    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the application runtime. */
    public void initialize()
    {
        try
        {
            // Setup JDK logging.
            final Handler handler = new FileHandler(
                new File(this.getStateDirectory(), this.getLogfileName()).
                getAbsolutePath());

            java.util.logging.Logger.getLogger("").addHandler(handler);
            java.util.logging.Logger.getLogger("").setLevel(Level.INFO);

            super.initialize();

            final Modules modules = ModelFactory.getModel().getModules();
            for(int i = modules.size() - 1; i >= 0; i--)
            {
                getLogger().debug(modules.getModule(i).getName() + '-' +
                    modules.getModule(i).getVersion());

            }

            // Register exception handler for uncaught exceptions.
            Thread.currentThread().setUncaughtExceptionHandler(
                this.getExceptionHandler());

            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    Thread.currentThread().setUncaughtExceptionHandler(
                        getExceptionHandler());

                }
            });

            // User interface initialization.
            this.getUserInterface();

            // Display a alpha-code warning and the license.
            final Object[] message = new Object[2];
            message[0] = EditorBundle.getWelcomeText(Locale.getDefault());
            message[1] = EditorBundle.getLicenseText(Locale.getDefault());

            JOptionPane.showMessageDialog(this.getUserInterface().getParent(),
                message);

        }
        catch(IOException e)
        {
            JOptionPane.showMessageDialog(this.getUserInterface().
                getParent(), e.getMessage());

            throw new org.jdtaus.core.container.
                InstantiationException(Editor.META.getIdentifier(), e);

        }
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Configured <code>Logger</code> implementation. */
    private transient Logger _dependency1;

    /** <code>Logger</code> implementation getter. */
    private Logger getLogger()
    {
        Logger ret = null;
        if(this._dependency1 != null)
        {
           ret = this._dependency1;
        }
        else
        {
            ret = (Logger) ContainerFactory.getContainer().
                getDependency(Editor.class,
                "Logger");

            if(ModelFactory.getModel().getModules().
                getImplementation(Editor.class.getName()).
                getDependencies().getDependency("Logger").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>UserInterface</code> implementation. */
    private transient UserInterface _dependency0;

    /** <code>UserInterface</code> implementation getter. */
    private UserInterface getUserInterface()
    {
        UserInterface ret = null;
        if(this._dependency0 != null)
        {
           ret = this._dependency0;
        }
        else
        {
            ret = (UserInterface) ContainerFactory.getContainer().
                getDependency(Editor.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(Editor.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /**
     * Property {@code directoryName}.
     * @serial
     */
    private java.lang.String _directoryName;

    /** <code>directoryName</code> property getter. */
    protected java.lang.String getDirectoryName()
    {
        return this._directoryName;
    }

    /**
     * Property {@code logfileName}.
     * @serial
     */
    private java.lang.String _logfileName;

    /** <code>logfileName</code> property getter. */
    protected java.lang.String getLogfileName()
    {
        return this._logfileName;
    }


    //--------------------------------------------------------------Properties--
    //--Editor------------------------------------------------------------------

    /** Base directory of the application. */
    private transient File baseDir;

    /** Handler for uncaught exceptions. */
    private transient Thread.UncaughtExceptionHandler exceptionHandler;

    public File getBaseDirectory()
    {
        if(this.baseDir == null)
        {
            this.baseDir = new File(System.getProperty("user.home") + '/' +
                this.getDirectoryName());

            if(!this.baseDir.exists())
            {
                this.baseDir.mkdirs();
            }
        }

        return this.baseDir;
    }

    public Thread.UncaughtExceptionHandler getExceptionHandler()
    {
        if(this.exceptionHandler == null)
        {
            this.exceptionHandler = new Thread.UncaughtExceptionHandler()
            {
                public void uncaughtException(final Thread t, final Throwable e)
                {
                    getLogger().fatal(e);

                    if(e instanceof Error)
                    {
                        JOptionPane.showMessageDialog(
                            getUserInterface().getParent(),

                            e.getMessage(),

                            EditorBundle.getUncaughtExceptionMessage(
                            Locale.getDefault()).format(new Object[] {
                            e.getClass().getName()}),

                            JOptionPane.ERROR_MESSAGE);

                    }
                    else
                    {
                        JOptionPane.showMessageDialog(
                            getUserInterface().getParent(),

                            EditorBundle.getBugReportMessage(
                            Locale.getDefault()).format(new Object[] {
                            getStateDirectory().getAbsolutePath()}),

                            EditorBundle.getUncaughtExceptionMessage(
                            Locale.getDefault()).format(new Object[] {
                            e.getClass().getName()}),

                            JOptionPane.ERROR_MESSAGE);
                    }
                }
            };
        }

        return this.exceptionHandler;
    }

    //------------------------------------------------------------------Editor--

}
