/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import org.jdtaus.banking.swing.BankleitzahlenVerzeichnisDialog;
import org.jdtaus.core.client.util.spi.DefaultUserInterface;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;

/**
 * GUI of the client application.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: EditorGUI.java 2627 2007-04-12 13:59:40Z schulte2005 $
 */
public class EditorGUI
    extends DefaultUserInterface
    implements ContainerInitializer
{

    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(EditorGUI.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Protected <code>EditorGUI</code> implementation constructor.
    * @param meta Implementation meta-data.
    */
    protected EditorGUI(final Implementation meta)
    {
        super(meta);
    }
    /** Protected <code>EditorGUI</code> dependency constructor.
    * @param meta Dependency meta-data.
    */
    protected EditorGUI(final Dependency meta)
    {
        super(meta);
    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Holds the singleton {@code PhysicalFileFrame} instance. */
    private PhysicalFileFrame component;

    public void initialize()
    {
        this.component = new PhysicalFileFrame();
        this.push(this.component);
        this.component.setVisible(true);
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.


    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.


    //--------------------------------------------------------------Properties--
    //--EditorGUI---------------------------------------------------------------

    /**
     * Dialog to display and search the {@code BankleitzahlenVerzeichnis}.
     * @serial
     */
    private BankleitzahlenVerzeichnisDialog blzDialog;

    /**
     * Dialog to display and edit a {@code Header}.
     * @serial
     */
    private HeaderDialog headerDialog;

    /**
     * Dialog to display and edit a {@code Transaction}.
     * @serial
     */
    private TransactionDialog transactionDialog;

    /**
     * Dialog for generating DTAUS files.
     * @serial
     */
    private GeneratorDialog generatorDialog;

    public BankleitzahlenVerzeichnisDialog getBankleitzahlenVerzeichnisDialog()
    {
        if(this.blzDialog == null)
        {
            this.blzDialog =
                new BankleitzahlenVerzeichnisDialog(this.component, true);

        }

        return this.blzDialog;
    }

    public HeaderDialog getHeaderDialog()
    {
        if(this.headerDialog == null)
        {
            this.headerDialog = new HeaderDialog(this.component, true);
        }

        return this.headerDialog;
    }

    public TransactionDialog getTransactionDialog()
    {
        if(this.transactionDialog == null)
        {
            this.transactionDialog =
                new TransactionDialog(this.component, true);
        }

        return this.transactionDialog;
    }

    public GeneratorDialog getGeneratorDialog()
    {
        if(this.generatorDialog == null)
        {
            this.generatorDialog =
                new GeneratorDialog(this.component, true);
        }

        return this.generatorDialog;
    }

    //---------------------------------------------------------------EditorGUI--

}
