/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.event.ActionEvent;
import java.text.ParseException;
import java.util.Locale;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.BankleitzahlInfo;
import org.jdtaus.banking.BankleitzahlenVerzeichnis;
import org.jdtaus.banking.swing.BankleitzahlenVerzeichnisDialog;
import org.jdtaus.core.client.util.IconSet;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.logging.spi.Logger;

/**
 * Swing action for searching a Bankleitzahl in the
 * {@code BankleitzahlenVerzeichnis}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: SearchBankleitzahlAction.java 2627 2007-04-12 13:59:40Z schulte2005 $
 */
public class SearchBankleitzahlAction extends AbstractAction
    implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(SearchBankleitzahlAction.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Protected <code>SearchBankleitzahlAction</code> implementation constructor.
    * @param meta Implementation meta-data.
    */
    protected SearchBankleitzahlAction(final Implementation meta)
    {
        super();
    }
    /** Protected <code>SearchBankleitzahlAction</code> dependency constructor.
    * @param meta Dependency meta-data.
    */
    protected SearchBankleitzahlAction(final Dependency meta)
    {
        super();
    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /**
     * Initializes the action.
     */
    public void initialize()
    {
        if(!this.nameless)
        {
            this.putValue(Action.NAME, SearchBankleitzahlActionBundle.
                getNameText(Locale.getDefault()));

        }

        this.putValue(Action.SHORT_DESCRIPTION, SearchBankleitzahlActionBundle.
            getDescriptionText(Locale.getDefault()));

        this.putValue(Action.SMALL_ICON, getIconSet().
            getIcon(IconSet.SEARCH, IconSet.SIZE_16X16,
            Locale.getDefault()));

    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency4;

    /** <code>Application</code> implementation getter. */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency4 != null)
        {
           ret = this._dependency4;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(SearchBankleitzahlAction.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(SearchBankleitzahlAction.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency4 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency3;

    /** <code>UserInterface</code> implementation getter. */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency3 != null)
        {
           ret = this._dependency3;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(SearchBankleitzahlAction.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(SearchBankleitzahlAction.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>BankleitzahlenVerzeichnis</code> implementation. */
    private transient BankleitzahlenVerzeichnis _dependency2;

    /** <code>BankleitzahlenVerzeichnis</code> implementation getter. */
    private BankleitzahlenVerzeichnis getBankleitzahlenVerzeichnis()
    {
        BankleitzahlenVerzeichnis ret = null;
        if(this._dependency2 != null)
        {
           ret = this._dependency2;
        }
        else
        {
            ret = (BankleitzahlenVerzeichnis) ContainerFactory.getContainer().
                getDependency(SearchBankleitzahlAction.class,
                "BankleitzahlenVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(SearchBankleitzahlAction.class.getName()).
                getDependencies().getDependency("BankleitzahlenVerzeichnis").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>IconSet</code> implementation. */
    private transient IconSet _dependency1;

    /** <code>IconSet</code> implementation getter. */
    private IconSet getIconSet()
    {
        IconSet ret = null;
        if(this._dependency1 != null)
        {
           ret = this._dependency1;
        }
        else
        {
            ret = (IconSet) ContainerFactory.getContainer().
                getDependency(SearchBankleitzahlAction.class,
                "IconSet");

            if(ModelFactory.getModel().getModules().
                getImplementation(SearchBankleitzahlAction.class.getName()).
                getDependencies().getDependency("IconSet").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Logger</code> implementation. */
    private transient Logger _dependency0;

    /** <code>Logger</code> implementation getter. */
    private Logger getLogger()
    {
        Logger ret = null;
        if(this._dependency0 != null)
        {
           ret = this._dependency0;
        }
        else
        {
            ret = (Logger) ContainerFactory.getContainer().
                getDependency(SearchBankleitzahlAction.class,
                "Logger");

            if(ModelFactory.getModel().getModules().
                getImplementation(SearchBankleitzahlAction.class.getName()).
                getDependencies().getDependency("Logger").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.


    //--------------------------------------------------------------Properties--
    //--SearchBankleitzahlAction------------------------------------------------

    /** The text component to update with the selected Bankleitzahl. */
    private JTextComponent target;

    /** Flag indicating if the action will have a name assigned. */
    private boolean nameless;

    /**
     * Creates a new {@code SearchBankleitzahlAction}.
     *
     * @param target a target text component to read a {@code Bankleitzahl} from
     * initializing the entity selection with and to fill with the selected
     * entity on closing the dialog or {@code null}.
     * @param nameless {@code true} for a nameless {@code Action}
     * implementation; {@code false} to provide a name for the action.
     */
    public SearchBankleitzahlAction(final JTextComponent target,
        final boolean nameless)
    {
        this(SearchBankleitzahlAction.META);
        this.target = target;
        this.nameless = nameless;
        this.initialize();
    }

    /**
     * Called when a user requests this action to be performed.
     * <p>The default action opens the {@code BankleitzahlenVerzeichnisDialog}
     * to let a user search the {@code BankleitzahlenVerzeichnis}. After the
     * dialog is closed the return status is checked for a selected
     * {@code Bankleitzahl}. If a user selected a {@code Bankleitzahl} the
     * {@code setText()} method of the target text component, if not
     * {@code null}, is called with the formatted value of the selected
     * {@code Bankleitzahl}.</p>
     */
    public void actionPerformed(final ActionEvent e)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                final BankleitzahlInfo info;
                final BankleitzahlenVerzeichnisDialog dialog =
                    getUserInterface().getBankleitzahlenVerzeichnisDialog();

                final Thread t = new Thread()
                {
                    public void run()
                    {
                        if(target != null)
                        {
                            try
                            {
                                dialog.setSelectedBank(
                                    getBankleitzahlenVerzeichnis().
                                    getHeadOffice(Bankleitzahl.parse(
                                    target.getText())));

                            }
                            catch(ParseException e)
                            {
                                getLogger().warn(SearchBankleitzahlActionBundle.
                                    getParseExceptionWarningMessage(
                                    Locale.getDefault()).
                                    format(new Object[] { e.getMessage() }));

                            }
                        }
                    }
                };

                t.setUncaughtExceptionHandler(getExceptionHandler());
                t.start();

                getUserInterface().push(dialog);
                dialog.setVisible(true);
                getUserInterface().pop();

                if(dialog.getReturnStatus() ==
                    BankleitzahlenVerzeichnisDialog.RET_OK)
                {
                    info = dialog.getSelectedBank();

                    if(info != null && target != null)
                    {
                        target.setText(info.getBankCode().
                            format(Bankleitzahl.ELECTRONIC_FORMAT));

                    }
                }
            }
        });
    }

    private Thread.UncaughtExceptionHandler getExceptionHandler()
    {
        return this.getApplication().getExceptionHandler();
    }

    //------------------------------------------------SearchBankleitzahlAction--

}
