/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Currency;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.UIManager;
import javax.swing.text.JTextComponent;
import org.jdtaus.banking.AlphaNumericText27;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Referenznummer11;
import org.jdtaus.banking.Textschluessel;
import org.jdtaus.banking.TextschluesselVerzeichnis;
import org.jdtaus.banking.dtaus.Transaction;
import org.jdtaus.banking.dtaus.spi.CurrencyDirectory;
import org.jdtaus.banking.swing.CurrencyRenderer;
import org.jdtaus.banking.swing.TextschluesselRenderer;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;

/**
 * Dialog to display and edit a {@code Transaction}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: TransactionDialog.java 2875 2007-04-14 07:23:52Z schulte2005 $
 */
public class TransactionDialog extends JDialog implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(TransactionDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Protected <code>TransactionDialog</code> implementation constructor.
    * @param meta Implementation meta-data.
    */
    protected TransactionDialog(final Implementation meta)
    {
        super();
    }
    /** Protected <code>TransactionDialog</code> dependency constructor.
    * @param meta Dependency meta-data.
    */
    protected TransactionDialog(final Dependency meta)
    {
        super();
    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

        this.updateModel();
        this.updateCurrencies();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.

    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency2;

    /** <code>UserInterface</code> implementation getter. */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency2 != null)
        {
           ret = this._dependency2;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>CurrencyDirectory</code> implementation. */
    private transient CurrencyDirectory _dependency1;

    /** <code>CurrencyDirectory</code> implementation getter. */
    private CurrencyDirectory getCurrencyDirectory()
    {
        CurrencyDirectory ret = null;
        if(this._dependency1 != null)
        {
           ret = this._dependency1;
        }
        else
        {
            ret = (CurrencyDirectory) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "CurrencyDirectory");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("CurrencyDirectory").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>TextschluesselVerzeichnis</code> implementation. */
    private transient TextschluesselVerzeichnis _dependency0;

    /** <code>TextschluesselVerzeichnis</code> implementation getter. */
    private TextschluesselVerzeichnis getTextschluesselVerzeichnis()
    {
        TextschluesselVerzeichnis ret = null;
        if(this._dependency0 != null)
        {
           ret = this._dependency0;
        }
        else
        {
            ret = (TextschluesselVerzeichnis) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "TextschluesselVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("TextschluesselVerzeichnis").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is generated by jdtaus-source-plugin.


    //--------------------------------------------------------------Properties--
    //--TransactionDialog-------------------------------------------------------

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /**
     * Creates new {@code TransactionDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public TransactionDialog(final Frame owner, final boolean modal)
    {
        super(owner, modal);
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the transaction displayed by the dialog.
     *
     * @return the transaction displayed by the dialog or {@code null} if
     * the dialog displays invalid data.
     */
    public Transaction getTransaction()
    {
        String str;
        AlphaNumericText27 txt;
        boolean valid = true;
        final Transaction t = new Transaction();
        final NumberFormat amountFmt = NumberFormat.getNumberInstance();
        final Currency cur = (Currency) this.currencyComboBox.getSelectedItem();

        try
        {
            str = this.executiveNameTextField.getText();
            t.setExecutiveName(AlphaNumericText27.parse(str));
            this.resetForeground(this.executiveNameLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.executiveNameLabel, Color.RED);
            this.executiveNameTextField.setText(AlphaNumericText27.
                normalize(this.executiveNameTextField.getText()));

            t.setExecutiveName(null);
            valid = false;
        }

        try
        {
            str = this.executiveExtTextField.getText();
            t.setExecutiveExt(AlphaNumericText27.parse(str));
            this.resetForeground(this.executiveExtLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.executiveExtLabel, Color.RED);
            t.setExecutiveExt(null);
            this.executiveExtTextField.setText(AlphaNumericText27.
                normalize(this.executiveExtTextField.getText()));

            valid = false;
        }

        try
        {
            str = this.executiveAccountTextField.getText();
            t.setExecutiveAccount(Kontonummer.parse(str));
            this.resetForeground(this.executiveAccountLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.executiveAccountLabel, Color.RED);
            t.setExecutiveAccount(null);
            valid = false;
        }

        try
        {
            str = this.executiveBankTextField.getText();
            t.setExecutiveBank(Bankleitzahl.parse(str));
            this.resetForeground(this.executiveBankLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.executiveBankLabel, Color.RED);
            t.setExecutiveBank(null);
            valid = false;
        }

        try
        {
            str = this.targetNameTextField.getText();
            t.setTargetName(AlphaNumericText27.parse(str));
            this.resetForeground(this.targetNameLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.targetNameLabel, Color.RED);
            this.targetNameTextField.setText(AlphaNumericText27.
                normalize(this.targetNameTextField.getText()));

            t.setTargetName(null);
            valid = false;
        }

        try
        {
            str = this.targetExtTextField.getText();
            t.setTargetExt(AlphaNumericText27.parse(str));
            this.resetForeground(this.targetExtLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.targetExtLabel, Color.RED);
            t.setTargetExt(null);
            this.targetExtTextField.setText(AlphaNumericText27.
                normalize(this.targetExtTextField.getText()));

            valid = false;
        }

        try
        {
            str = this.targetAccountTextField.getText();
            t.setTargetAccount(Kontonummer.parse(str));
            this.resetForeground(this.targetAccountLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.targetAccountLabel, Color.RED);
            t.setTargetAccount(null);
            valid = false;
        }

        try
        {
            str = this.targetBankTextField.getText();
            t.setTargetBank(Bankleitzahl.parse(str));
            this.resetForeground(this.targetBankLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.targetBankLabel, Color.RED);
            t.setTargetBank(null);
            valid = false;
        }

        if(cur == null)
        {
            this.setForeground(this.amountLabel, Color.RED);
            valid = false;
        }
        else
        {
            t.setCurrency(cur);

            amountFmt.setMaximumFractionDigits(
                cur.getDefaultFractionDigits());

            amountFmt.setMinimumFractionDigits(
                cur.getDefaultFractionDigits());

            try
            {
                str = this.amountTextField.getText();
                t.setAmount(new BigDecimal(amountFmt.parse(str).doubleValue()).
                    multiply(new BigDecimal(Math.pow(10,
                    cur.getDefaultFractionDigits()))).toBigInteger());

                this.resetForeground(this.amountLabel);
            }
            catch(ParseException e)
            {
                this.setForeground(this.amountLabel, Color.RED);
                valid = false;
            }
        }

        try
        {
            str = this.referenceTextField.getText();
            t.setReference(Referenznummer11.parse(str));
            this.resetForeground(this.referenceLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.referenceLabel, Color.RED);
            valid = false;
        }

        try
        {
            str = this.descriptionTextField1.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel1);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel1, Color.RED);
            this.descriptionTextField1.setText(AlphaNumericText27.
                normalize(this.descriptionTextField1.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField2.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel2);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel2, Color.RED);
            this.descriptionTextField2.setText(AlphaNumericText27.
                normalize(this.descriptionTextField2.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField3.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel3);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel3, Color.RED);
            this.descriptionTextField3.setText(AlphaNumericText27.
                normalize(this.descriptionTextField3.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField4.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel4);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel4, Color.RED);
            this.descriptionTextField4.setText(AlphaNumericText27.
                normalize(this.descriptionTextField4.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField5.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel5);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel5, Color.RED);
            this.descriptionTextField5.setText(AlphaNumericText27.
                normalize(this.descriptionTextField5.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField6.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel6);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel6, Color.RED);
            this.descriptionTextField6.setText(AlphaNumericText27.
                normalize(this.descriptionTextField6.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField7.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel7);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel7, Color.RED);
            this.descriptionTextField7.setText(AlphaNumericText27.
                normalize(this.descriptionTextField7.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField8.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel8);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel8, Color.RED);
            this.descriptionTextField8.setText(AlphaNumericText27.
                normalize(this.descriptionTextField8.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField9.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel9);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel9, Color.RED);
            this.descriptionTextField9.setText(AlphaNumericText27.
                normalize(this.descriptionTextField9.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField10.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel10);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel10, Color.RED);
            this.descriptionTextField10.setText(AlphaNumericText27.
                normalize(this.descriptionTextField10.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField11.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel11);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel11, Color.RED);
            this.descriptionTextField11.setText(AlphaNumericText27.
                normalize(this.descriptionTextField11.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField12.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel12);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel12, Color.RED);
            this.descriptionTextField12.setText(AlphaNumericText27.
                normalize(this.descriptionTextField12.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField13.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel13);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel13, Color.RED);
            this.descriptionTextField13.setText(AlphaNumericText27.
                normalize(this.descriptionTextField13.getText()));

            valid = false;
        }

        try
        {
            str = this.descriptionTextField14.getText();
            txt = AlphaNumericText27.parse(str);
            if(txt.format().length() > 0)
            {
                t.getDescription().addDescription(txt);
            }

            this.resetForeground(this.descriptionLabel14);
        }
        catch(ParseException e)
        {
            this.setForeground(this.descriptionLabel14, Color.RED);
            this.descriptionTextField14.setText(AlphaNumericText27.
                normalize(this.descriptionTextField14.getText()));

            valid = false;
        }

        t.setType((Textschluessel) this.textschluesselComboBox.
            getSelectedItem());

        if(t.getType() == null)
        {
            this.setForeground(this.textschluesselLabel, Color.RED);
            valid = false;
        }
        else
        {
            this.resetForeground(this.textschluesselLabel);
        }

        return valid ? t : null;
    }

    /**
     * Sets the transaction to be displayed by the dialog.
     *
     * @param value the transaction to be displayed by the dialog or
     * {@code null} to reset the dialog display.
     */
    public void setTransaction(final Transaction value)
    {
        AlphaNumericText27 txt;
        JTextComponent txtComp;

        for(JLabel label : this.colors.keySet())
        {
            this.resetForeground(label);
        }

        if(value == null)
        {
            this.executiveAccountTextField.setText(null);
            this.executiveBankTextField.setText(null);
            this.executiveExtTextField.setText(null);
            this.executiveNameTextField.setText(null);
            this.targetAccountTextField.setText(null);
            this.targetBankTextField.setText(null);
            this.targetExtTextField.setText(null);
            this.targetNameTextField.setText(null);
            this.amountTextField.setText(null);
            this.referenceTextField.setText(null);
            this.textschluesselComboBox.setSelectedItem(null);
            this.currencyComboBox.setSelectedItem(null);
            this.descriptionTextField1.setText(null);
            this.descriptionTextField2.setText(null);
            this.descriptionTextField3.setText(null);
            this.descriptionTextField4.setText(null);
            this.descriptionTextField5.setText(null);
            this.descriptionTextField6.setText(null);
            this.descriptionTextField7.setText(null);
            this.descriptionTextField8.setText(null);
            this.descriptionTextField9.setText(null);
            this.descriptionTextField10.setText(null);
            this.descriptionTextField11.setText(null);
            this.descriptionTextField12.setText(null);
            this.descriptionTextField13.setText(null);
            this.descriptionTextField14.setText(null);
        }
        else
        {
            BigDecimal amount = new BigDecimal(value.getAmount());
            final Currency cur = value.getCurrency();
            NumberFormat amountFmt = NumberFormat.getNumberInstance();

            if(cur != null)
            {
                amountFmt.setMaximumFractionDigits(
                    cur.getDefaultFractionDigits());

                amountFmt.setMinimumFractionDigits(
                    cur.getDefaultFractionDigits());

                if(amount != null)
                {
                    amount = amount.divide(new BigDecimal(Math.pow(10,
                        cur.getDefaultFractionDigits())),
                        cur.getDefaultFractionDigits(),
                        RoundingMode.HALF_UP);

                }
            }

            this.currencyComboBox.setSelectedItem(cur);

            this.amountTextField.setText(amount != null ?
                amountFmt.format(amount) : null);

            this.executiveAccountTextField.setText(
                value.getExecutiveAccount() != null
                ? value.getExecutiveAccount().format(
                Kontonummer.ELECTRONIC_FORMAT) : null);

            this.executiveBankTextField.setText(
                value.getExecutiveBank() != null
                ? value.getExecutiveBank().
                format(Bankleitzahl.ELECTRONIC_FORMAT) : null);

            this.executiveNameTextField.setText(
                value.getExecutiveName() != null
                ? value.getExecutiveName().format().trim() : null);


            this.executiveExtTextField.setText(
                value.getExecutiveExt() != null
                ? value.getExecutiveExt().format().trim() : null);

            this.targetAccountTextField.setText(
                value.getTargetAccount() != null
                ? value.getTargetAccount().format(
                Kontonummer.ELECTRONIC_FORMAT) : null);

            this.targetBankTextField.setText(
                value.getTargetBank() != null
                ? value.getTargetBank().format(
                Bankleitzahl.ELECTRONIC_FORMAT) : null);

            this.targetNameTextField.setText(
                value.getTargetName() != null
                ? value.getTargetName().format().trim() : null);

            this.targetExtTextField.setText(
                value.getTargetExt() != null
                ? value.getTargetExt().format().trim() : null);

            this.referenceTextField.setText(value.getReference() != null
                ? value.getReference().format(Referenznummer11.ELECTRONIC_FORMAT)
                : null);

            this.textschluesselComboBox.setSelectedItem(value.getType());

            for(int i = 14; i > 0; i--)
            {
                txtComp = this.getDescriptionComponent(i);
                if(i - 1 < value.getDescription().getDescriptionCount())
                {
                    txt = value.getDescription().getDescription(i - 1);
                    txtComp.setText(txt.format().trim());
                }
                else
                {
                    txtComp.setText(null);
                }
            }
        }
    }

    /**
     * Flag indicating if the dialog supports debit {@code Textschluessel}.
     *
     * @return {@code true} if the dialog supports debit {@code Textschluessel};
     * {@code false} if not.
     */
    public boolean isDebit()
    {
        return this.debit;
    }

    /**
     * Setter for property {@code debit}.
     *
     * @param debit {@code true} if the dialog supports debit
     * {@code Textschluessel}; {@code false} if not.
     */
    public void setDebit(final boolean debit)
    {
        this.firePropertyChange("debit", this.debit, debit);
        this.debit = debit;
        this.updateModel();
    }

    /**
     * Flag indicating if the dialog supports remittance {@code Textschluessel}.
     *
     * @return {@code true} if the dialog supports remittance
     * {@code Textschluessel}; {@code false} if not.
     */
    public boolean isRemittance()
    {
        return this.remittance;
    }

    /**
     * Setter for property {@code remittance}.
     *
     * @param remittance {@code true} if the dialog supports remittance
     * {@code Textschluessel}; {@code false} if not.
     */
    public void setRemittance(final boolean remittance)
    {
        this.firePropertyChange("remittance", this.remittance, remittance);
        this.remittance = remittance;
        this.updateModel();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        transactionPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        executiveNameLabel = new javax.swing.JLabel();
        executiveNameTextField = new javax.swing.JTextField();
        executiveExtLabel = new javax.swing.JLabel();
        executiveExtTextField = new javax.swing.JTextField();
        executiveBankLabel = new javax.swing.JLabel();
        executiveBankTextField = new javax.swing.JTextField();
        executiveAccountLabel = new javax.swing.JLabel();
        executiveAccountTextField = new javax.swing.JTextField();
        targetNameLabel = new javax.swing.JLabel();
        targetNameTextField = new javax.swing.JTextField();
        targetExtLabel = new javax.swing.JLabel();
        targetExtTextField = new javax.swing.JTextField();
        targetBankLabel = new javax.swing.JLabel();
        targetBankTextField = new javax.swing.JTextField();
        targetAccountLabel = new javax.swing.JLabel();
        targetAccountTextField = new javax.swing.JTextField();
        amountLabel = new javax.swing.JLabel();
        amountTextField = new javax.swing.JTextField();
        executiveBankleitzahlButton = new javax.swing.JButton();
        targetBankleitzahlButton = new javax.swing.JButton();
        currencyComboBox = new javax.swing.JComboBox();
        referenceLabel = new javax.swing.JLabel();
        referenceTextField = new javax.swing.JTextField();
        jPanel2 = new javax.swing.JPanel();
        descriptionTextField1 = new javax.swing.JTextField();
        descriptionLabel2 = new javax.swing.JLabel();
        descriptionTextField2 = new javax.swing.JTextField();
        descriptionLabel1 = new javax.swing.JLabel();
        descriptionLabel3 = new javax.swing.JLabel();
        descriptionTextField3 = new javax.swing.JTextField();
        descriptionLabel4 = new javax.swing.JLabel();
        descriptionTextField4 = new javax.swing.JTextField();
        descriptionLabel5 = new javax.swing.JLabel();
        descriptionTextField5 = new javax.swing.JTextField();
        descriptionLabel6 = new javax.swing.JLabel();
        descriptionTextField6 = new javax.swing.JTextField();
        descriptionLabel7 = new javax.swing.JLabel();
        descriptionTextField7 = new javax.swing.JTextField();
        descriptionLabel8 = new javax.swing.JLabel();
        descriptionTextField8 = new javax.swing.JTextField();
        descriptionLabel9 = new javax.swing.JLabel();
        descriptionTextField9 = new javax.swing.JTextField();
        descriptionLabel10 = new javax.swing.JLabel();
        descriptionTextField10 = new javax.swing.JTextField();
        descriptionLabel11 = new javax.swing.JLabel();
        descriptionLabel12 = new javax.swing.JLabel();
        descriptionLabel13 = new javax.swing.JLabel();
        descriptionLabel14 = new javax.swing.JLabel();
        descriptionTextField11 = new javax.swing.JTextField();
        descriptionTextField12 = new javax.swing.JTextField();
        descriptionTextField13 = new javax.swing.JTextField();
        descriptionTextField14 = new javax.swing.JTextField();
        textschluesselLabel = new javax.swing.JLabel();
        textschluesselComboBox = new javax.swing.JComboBox();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(TransactionDialogBundle.getTitleText(Locale.getDefault()));
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cancelButtonActionPerformed(evt);
            }
        });

        transactionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(TransactionDialogBundle.getTransactionPanelTitleText(Locale.getDefault())));
        executiveNameLabel.setText(TransactionDialogBundle.getExecutiveNameLabelText(Locale.getDefault()));

        executiveNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        executiveExtLabel.setText(TransactionDialogBundle.getExecutiveName2LabelText(Locale.getDefault()));

        executiveExtTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        executiveBankLabel.setText(TransactionDialogBundle.getExecutiveBankLabelText(Locale.getDefault()));

        executiveBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        executiveAccountLabel.setText(TransactionDialogBundle.getExecutiveAccountLabelText(Locale.getDefault()));

        executiveAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        targetNameLabel.setText(TransactionDialogBundle.getTargetNameLabelText(Locale.getDefault()));

        targetNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        targetExtLabel.setText(TransactionDialogBundle.getTargetName2LabelText(Locale.getDefault()));

        targetExtTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        targetBankLabel.setText(TransactionDialogBundle.getTargetBankLabelText(Locale.getDefault()));

        targetBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        targetAccountLabel.setText(TransactionDialogBundle.getTargetAccountLabelText(Locale.getDefault()));

        targetAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        amountLabel.setText(TransactionDialogBundle.getAmountLabelText(Locale.getDefault()));

        executiveBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.executiveBankTextField, true));
        executiveBankleitzahlButton.setBorder(null);
        executiveBankleitzahlButton.setBorderPainted(false);
        executiveBankleitzahlButton.setContentAreaFilled(false);
        executiveBankleitzahlButton.setDefaultCapable(false);
        executiveBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        executiveBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        executiveBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        targetBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.targetBankTextField, true));
        targetBankleitzahlButton.setBorder(null);
        targetBankleitzahlButton.setBorderPainted(false);
        targetBankleitzahlButton.setContentAreaFilled(false);
        targetBankleitzahlButton.setDefaultCapable(false);
        targetBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        targetBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        targetBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        currencyComboBox.setRenderer(new CurrencyRenderer());

        referenceLabel.setText(TransactionDialogBundle.getReferenceLabelText(Locale.getDefault()));

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(referenceLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveExtLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetExtLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(amountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(amountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 87, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(currencyComboBox, 0, 93, Short.MAX_VALUE))
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(targetExtTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(executiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(executiveNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(executiveExtTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(targetBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(targetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(16, 16, 16)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveNameLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveExtLabel)
                    .add(executiveExtTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveBankLabel)
                    .add(executiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveAccountLabel)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetNameLabel)
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetExtLabel)
                    .add(targetExtTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetBankLabel)
                    .add(targetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(targetBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetAccountLabel)
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(amountLabel)
                    .add(amountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(currencyComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(referenceLabel)
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(156, Short.MAX_VALUE))
        );

        descriptionTextField1.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel2.setText(TransactionDialogBundle.getDescriptionLabel2Text(Locale.getDefault()));

        descriptionTextField2.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel1.setText(TransactionDialogBundle.getDescriptionLabel1Text(Locale.getDefault()));

        descriptionLabel3.setText(TransactionDialogBundle.getDescriptionLabel3Text(Locale.getDefault()));

        descriptionTextField3.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel4.setText(TransactionDialogBundle.getDescriptionLabel4Text(Locale.getDefault()));

        descriptionTextField4.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel5.setText(TransactionDialogBundle.getDescriptionLabel5Text(Locale.getDefault()));

        descriptionTextField5.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel6.setText(TransactionDialogBundle.getDescriptionLabel6Text(Locale.getDefault()));

        descriptionTextField6.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel7.setText(TransactionDialogBundle.getDescriptionLabel7Text(Locale.getDefault()));

        descriptionTextField7.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel8.setText(TransactionDialogBundle.getDescriptionLabel8Text(Locale.getDefault()));

        descriptionTextField8.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel9.setText(TransactionDialogBundle.getDescriptionLabel9Text(Locale.getDefault()));

        descriptionTextField9.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel10.setText(TransactionDialogBundle.getDescriptionLabel10Text(Locale.getDefault()));

        descriptionTextField10.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel11.setText(TransactionDialogBundle.getDescriptionLabel11Text(Locale.getDefault()));

        descriptionLabel12.setText(TransactionDialogBundle.getDescriptionLabel12Text(Locale.getDefault()));

        descriptionLabel13.setText(TransactionDialogBundle.getDescriptionLabel13Text(Locale.getDefault()));

        descriptionLabel14.setText(TransactionDialogBundle.getDescriptionLabel14Text(Locale.getDefault()));

        descriptionTextField11.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionTextField12.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionTextField13.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionTextField14.setColumns(AlphaNumericText27.MAX_LENGTH);

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(descriptionLabel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
                    .add(descriptionLabel4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
                    .add(descriptionLabel5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel7, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 128, Short.MAX_VALUE)
                    .add(descriptionLabel8, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel9, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel10, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel11, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel12, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel13, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel14, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(descriptionTextField14, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField13, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField12, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField11, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField10, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField9, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField8, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField7, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(descriptionTextField1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel1)
                    .add(descriptionTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel2))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel3)
                    .add(descriptionTextField3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel4)
                    .add(descriptionTextField4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel5)
                    .add(descriptionTextField5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel6)
                    .add(descriptionTextField6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel7)
                    .add(descriptionTextField7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField8, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel8))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField9, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel9))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel10))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField11, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel11))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField12, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel12))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField13, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel13))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel14))
                .addContainerGap(65, Short.MAX_VALUE))
        );

        textschluesselLabel.setText(TransactionDialogBundle.getTextschluesselLabelText(Locale.getDefault()));

        textschluesselComboBox.setRenderer(new TextschluesselRenderer());

        org.jdesktop.layout.GroupLayout transactionPanelLayout = new org.jdesktop.layout.GroupLayout(transactionPanel);
        transactionPanel.setLayout(transactionPanelLayout);
        transactionPanelLayout.setHorizontalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, transactionPanelLayout.createSequentialGroup()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(transactionPanelLayout.createSequentialGroup()
                        .add(textschluesselLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(textschluesselComboBox, 0, 616, Short.MAX_VALUE)))
                .addContainerGap())
        );
        transactionPanelLayout.setVerticalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionPanelLayout.createSequentialGroup()
                .add(transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(textschluesselLabel)
                    .add(textschluesselComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(cancelButton))
                    .add(transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(cancelButton)
                    .add(okButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_okButtonActionPerformed
    {
        if(this.getTransaction() != null)
        {
            this.doClose(RET_OK);
        }
        else
        {
            UIManager.getLookAndFeel().provideErrorFeedback(
                getUserInterface().getParent());

        }
    }//GEN-LAST:event_okButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
    {
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
    {
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_closeDialog

    private void doClose(int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    private void setForeground(final JLabel label, final Color color)
    {
        if(!this.colors.containsKey(label))
        {
            this.colors.put(label, label.getForeground());
        }
        label.setForeground(color);
    }

    private void resetForeground(final JLabel label)
    {
        if(this.colors.containsKey(label))
        {
            label.setForeground(this.colors.get(label));
        }
    }

    private JTextComponent getDescriptionComponent(final int index)
    {
        switch(index)
        {
            case 1: return this.descriptionTextField1;
            case 2: return this.descriptionTextField2;
            case 3: return this.descriptionTextField3;
            case 4: return this.descriptionTextField4;
            case 5: return this.descriptionTextField5;
            case 6: return this.descriptionTextField6;
            case 7: return this.descriptionTextField7;
            case 8: return this.descriptionTextField8;
            case 9: return this.descriptionTextField9;
            case 10: return this.descriptionTextField10;
            case 11: return this.descriptionTextField11;
            case 12: return this.descriptionTextField12;
            case 13: return this.descriptionTextField13;
            case 14: return this.descriptionTextField14;
            default:
                throw new IllegalArgumentException(Integer.toString(index));

        }
    }

    private void updateModel()
    {
        final Textschluessel[] keys = getTextschluesselVerzeichnis().
            search(isDebit(), isRemittance());

        Arrays.sort(keys, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof Textschluessel;
                assert o2 instanceof Textschluessel;

                final Textschluessel t1 = (Textschluessel) o1;
                final Textschluessel t2 = (Textschluessel) o2;

                int ret = t1.getKey() > t2.getKey() ? 1 :
                    t1.getKey() < t2.getKey() ? -1 : 0;

                if(ret == 0)
                {
                    ret = t1.getExtension() > t2.getExtension() ? 1:
                        t1.getExtension() < t2.getExtension() ? -1 : 0;

                }

                return ret;
            }
        });

        final Object prevSelection =
            textschluesselComboBox.getSelectedItem();

        textschluesselComboBox.removeAllItems();
        for(int i = 0; i < keys.length; i++)
        {
            textschluesselComboBox.addItem(keys[i]);
        }

        textschluesselComboBox.setSelectedItem(prevSelection);
    }

    private void updateCurrencies()
    {
        final Currency[] currencies = getCurrencyDirectory().
            getCurrencies();

        Arrays.sort(currencies, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof Currency;
                assert o2 instanceof Currency;

                return ((Currency) o1).getCurrencyCode().
                    compareTo(((Currency) o2).getCurrencyCode());

            }
        });

        final Object prevSelection = currencyComboBox.getSelectedItem();
        currencyComboBox.removeAllItems();
        for(int i = 0; i < currencies.length; i++)
        {
            currencyComboBox.addItem(currencies[i]);
        }

        currencyComboBox.setSelectedItem(prevSelection);
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel amountLabel;
    private javax.swing.JTextField amountTextField;
    private javax.swing.JButton cancelButton;
    private javax.swing.JComboBox currencyComboBox;
    private javax.swing.JLabel descriptionLabel1;
    private javax.swing.JLabel descriptionLabel10;
    private javax.swing.JLabel descriptionLabel11;
    private javax.swing.JLabel descriptionLabel12;
    private javax.swing.JLabel descriptionLabel13;
    private javax.swing.JLabel descriptionLabel14;
    private javax.swing.JLabel descriptionLabel2;
    private javax.swing.JLabel descriptionLabel3;
    private javax.swing.JLabel descriptionLabel4;
    private javax.swing.JLabel descriptionLabel5;
    private javax.swing.JLabel descriptionLabel6;
    private javax.swing.JLabel descriptionLabel7;
    private javax.swing.JLabel descriptionLabel8;
    private javax.swing.JLabel descriptionLabel9;
    private javax.swing.JTextField descriptionTextField1;
    private javax.swing.JTextField descriptionTextField10;
    private javax.swing.JTextField descriptionTextField11;
    private javax.swing.JTextField descriptionTextField12;
    private javax.swing.JTextField descriptionTextField13;
    private javax.swing.JTextField descriptionTextField14;
    private javax.swing.JTextField descriptionTextField2;
    private javax.swing.JTextField descriptionTextField3;
    private javax.swing.JTextField descriptionTextField4;
    private javax.swing.JTextField descriptionTextField5;
    private javax.swing.JTextField descriptionTextField6;
    private javax.swing.JTextField descriptionTextField7;
    private javax.swing.JTextField descriptionTextField8;
    private javax.swing.JTextField descriptionTextField9;
    private javax.swing.JLabel executiveAccountLabel;
    private javax.swing.JTextField executiveAccountTextField;
    private javax.swing.JLabel executiveBankLabel;
    private javax.swing.JTextField executiveBankTextField;
    private javax.swing.JButton executiveBankleitzahlButton;
    private javax.swing.JLabel executiveExtLabel;
    private javax.swing.JTextField executiveExtTextField;
    private javax.swing.JLabel executiveNameLabel;
    private javax.swing.JTextField executiveNameTextField;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel referenceLabel;
    private javax.swing.JTextField referenceTextField;
    private javax.swing.JLabel targetAccountLabel;
    private javax.swing.JTextField targetAccountTextField;
    private javax.swing.JLabel targetBankLabel;
    private javax.swing.JTextField targetBankTextField;
    private javax.swing.JButton targetBankleitzahlButton;
    private javax.swing.JLabel targetExtLabel;
    private javax.swing.JTextField targetExtTextField;
    private javax.swing.JLabel targetNameLabel;
    private javax.swing.JTextField targetNameTextField;
    private javax.swing.JComboBox textschluesselComboBox;
    private javax.swing.JLabel textschluesselLabel;
    private javax.swing.JPanel transactionPanel;
    // End of variables declaration//GEN-END:variables

    /**
     * Return status code.
     * @serial
     */
    private int returnStatus = RET_CANCEL;

    /**
     * Flag indicating if the dialog supports debit {@code Textschluessel}.
     * @serial
     */
    private boolean debit;

    /**
     * Flag indicating if the dialog supports remittance {@code Textschluessel}.
     * @serial
     */
    private boolean remittance;

    /**
     * Caches label colors.
     * @serial
     */
    private Map<JLabel, Color> colors = new HashMap(50);

    //-------------------------------------------------------TransactionDialog--

}
